/*
 * Copyright (c) 2012 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is confidential property of Nordic Semiconductor. The use,
 * copying, transfer or disclosure of such information is prohibited except by express written
 * agreement with Nordic Semiconductor.
 *
 */

/**@file
 *
 * @defgroup ble_sdk_srv_whts_c   Water Heater Temp Service Client
 * @{
 * @ingroup  ble_sdk_srv
 * @brief    Water Heater Temp Service Client module.
 *
 * @details  This module contains the APIs and types exposed by the Water Heater Temp Service Client
 *           module. These APIs and types can be used by the application to perform discovery of
 *           Water Heater Temp Service at the peer and interact with it.
 *
 * @warning  Currently this module only has support for Water Heater Temp Measurement characteristic. This
 *           means that it will be able to enable notification of the characteristic at the peer and
 *           be able to receive Water Heater Temp Measurement notifications from the peer.
 *           When a Water Heater Temp Measurement is received, this module will decode only the
 *           Water Heater Temp Measurement Value (both 8 bit and 16 bit) field from it and provide it to
 *           the application.
 *
 * @note     The application must propagate BLE stack events to this module by calling
 *           ble_whts_c_on_ble_evt().
 *
 */

#ifndef BLE_WHTS_C_H__
#define BLE_WHTS_C_H__

#include <stdint.h>
#include "ble.h"

#define WHTS_BASE_UUID {{ 0x1b, 0xc5, 0xd5, 0xa5, 0x02, 0x00, 0x78, 0xa3, \
        0xe5, 0x11, 0xac, 0xb8, 0xa0, 0x49, 0x18, 0x02 } }
#define BLE_UUID_WHTS_SERVICE  0x0001     /**< The UUID of the custom service          */
#define BLE_UUID_WHTS_SENSOR   0x0002     /**< The UUID of the sensor characteristic   */
#define BLE_UUID_WHTS_LED      0x0003     /**< The UUID of the LED ctrl characteristic */
#define WHTS_SERVICE_UUID_TYPE BLE_UUID_TYPE_VENDOR_BEGIN /**< UUID type for Eaton WHTS */

/**
 * @defgroup whts_c_enums Enumerations
 * @{
 */

/**@brief whts Client event type. */
typedef enum
{
    BLE_WHTS_C_EVT_DISCOVERY_COMPLETE = 1,  /**< Event indicating that the Water Heater Temp Service has been discovered at the peer. */
    BLE_WHTS_C_EVT_WHT_NOTIFICATION         /**< Event indicating that a notification of the Water Heater Temp Measurement characteristic has been received from the peer. */
} ble_whts_c_evt_type_t;

/** @} */

/**
 * @defgroup whts_c_structs Structures
 * @{
 */

/**@brief Structure containing the Water Heater Temp measurement received from the peer. */
typedef struct
{
    uint16_t temperature;  /**< Water Heater Temp Value. */
} ble_wht_t;

/**@brief Water Heater Temp Event structure. */
typedef struct
{
    ble_whts_c_evt_type_t evt_type;  /**< Type of the event. */
    union
    {
        ble_wht_t wht;  /**< Water Heater Temp meas received. This will be filled if the evt_type is @ref BLE_whts_C_EVT_HRM_NOTIFICATION. */
    } params;
} ble_whts_c_evt_t;

/** @} */

/**
 * @defgroup whts_c_types Types
 * @{
 */

// Forward declaration of the ble_bas_t type.
typedef struct ble_whts_c_s ble_whts_c_t;

/**@brief   Event handler type.
 *
 * @details This is the type of the event handler that should be provided by the application
 *          of this module in order to receive events.
 */
typedef void (* ble_whts_c_evt_handler_t) (ble_whts_c_t * p_ble_whts_c, ble_whts_c_evt_t * p_evt);

/** @} */

/**
 * @addtogroup whts_c_structs
 * @{
 */

/**@brief Water Heater Temp Client structure.
 */
struct ble_whts_c_s
{
    uint8_t                  uuid_type;        /**< UUID type. */
    uint16_t                 conn_handle;      /**< Connection handle as provided by the SoftDevice. */
    uint16_t                 wht_cccd_handle;  /**< Handle of the CCCD of the Water Heater Temp Measurement characteristic. */
    uint16_t                 wht_handle;       /**< Handle of the Water Heater Temp Measurement characteristic as provided by the SoftDevice. */
    uint16_t                 led_handle;       /**< Handle of the Water Heater LED characteristic as provided by the SoftDevice */
    ble_whts_c_evt_handler_t evt_handler;      /**< Application event handler to be called when there is an event related to the Water Heater Temp service. */
};

/**@brief Water Heater Temp Client initialization structure.
 */
typedef struct
{
    ble_whts_c_evt_handler_t evt_handler;  /**< Event handler to be called by the Water Heater Temp Client module whenever there is an event related to the Water Heater Temp Service. */
} ble_whts_c_init_t;

/** @} */

/**
 * @defgroup whts_c_functions Functions
 * @{
 */

/**@brief     Function for initializing the Water Heater Temp client module.
 *
 * @details   This function will register with the DB Discovery module. There it
 *            registers for the Water Heater Temp Service. Doing so will make the DB Discovery
 *            module look for the presence of a Water Heater Temp Service instance at the peer when a
 *            discovery is started.
 *
 * @param[in] p_ble_whts_c      Pointer to the Water Heater Temp client structure.
 * @param[in] p_ble_whts_c_init Pointer to the Water Heater Temp initialization structure containing the
 *                             initialization information.
 *
 * @retval    NRF_SUCCESS On successful initialization. Otherwise an error code. This function
 *                        propagates the error code returned by the Database Discovery module API
 *                        @ref ble_db_discovery_evt_register.
 */
uint32_t ble_whts_c_init(ble_whts_c_t * p_ble_whts_c, ble_whts_c_init_t * p_ble_whts_c_init);

/**@brief     Function for handling BLE events from the SoftDevice.
 *
 * @details   This function will handle the BLE events received from the SoftDevice. If a BLE
 *            event is relevant to the Water Heater Temp Client module, then it uses it to update
 *            interval variables and, if necessary, send events to the application.
 *
 * @param[in] p_ble_whts_c Pointer to the Water Heater Temp client structure.
 * @param[in] p_ble_evt   Pointer to the BLE event.
 */
void ble_whts_c_on_ble_evt(ble_whts_c_t * p_ble_whts_c, const ble_evt_t * p_ble_evt);


/**@brief   Function for requesting the peer to start sending notification of Water Heater Temp
 *          Measurement.
 *
 * @details This function will enable to notification of the Water Heater Temp Measurement at the peer
 *          by writing to the CCCD of the Water Heater Temp Measurement Characteristic.
 *
 * @param   p_ble_whts_c Pointer to the Water Heater Temp client structure.
 *
 * @retval  NRF_SUCCESS If the SoftDevice has been requested to write to the CCCD of the peer.
 *                      Otherwise, an error code. This function propagates the error code returned 
 *                      by the SoftDevice API @ref sd_ble_gattc_write.
 */
uint32_t ble_whts_c_wht_notif_enable(ble_whts_c_t * p_ble_whts_c);

/** @} */ // End tag for Function group.

#endif // BLE_WHTS_C_H__

/** @} */ // End tag for the file.
